@@ -12,9 +12,10 @@ from . import rel
 from .loaders import csv
 from . import analysis
 from . import tools
+from .util.otel_configuration import configure_otel
 from snowflake.snowpark import Session
 from .errors import RAIException, handle_missing_integration
-from .environments import runtime_env, SessionEnvironment, SnowbookEnvironment
+from .environments import runtime_env, SessionEnvironment
 from relationalai.tools.constants import USE_DIRECT_ACCESS, Generation
 
 import __main__
@@ -135,24 +136,7 @@ def Model(
                 format=format,
             )
 
-            # allow to enable the OpenTelemetry handler only for internal snowflake accounts
-            # Skip OTEL handling entirely for warehouse environments, 
-            # skipping is possible because it is not set up in the warehouse environment
-            if isinstance(runtime_env, SnowbookEnvironment) and runtime_env.runner == "warehouse":
-                pass
-            elif (
-                enable_otel_handler
-                and platform == "snowflake"
-                and config.is_internal_account()
-            ):
-                from .util.otel_handler import enable_otel_export
-                enable_otel_export(
-                    cast(snowflake.Resources, model._client.resources),
-                    config.get('rai_app_name', 'RELATIONALAI')
-                )
-            else:
-                from .util.otel_handler import disable_otel_handling
-                disable_otel_handling()
+            configure_otel(enable_otel_handler, config, model._client.resources)
 
         else:
             raise Exception(f"Unknown platform: {platform}")
