@@ -12,7 +12,7 @@ from pandas import DataFrame
 from relationalai import debugging
 from relationalai.clients.util import poll_with_specified_overhead
 
-from ..errors import EngineNotFoundException, RAIException
+from ..errors import EngineNotFoundException, RAIException, AzureUnsupportedQueryTimeoutException
 from ..rel_utils import assert_no_problems
 from ..loaders.loader import emit_delete_import, import_file, list_available_resources
 from .config import Config
@@ -227,10 +227,13 @@ class Resources(ResourcesBase):
     def _exec(self, code:str, params:List[Any]|Any|None = None, raw=False, help=True):
         raise Exception("Azure doesn't support _exec")
 
-    def exec_lqp(self, database: str, engine: str | None, raw_code: bytes, readonly=True, *, inputs: Dict | None = None, nowait_durable=False, headers: Dict | None = None, bypass_index=False):
+    def exec_lqp(self, database: str, engine: str | None, raw_code: bytes, readonly=True, *, inputs: Dict | None = None, nowait_durable=False, headers: Dict | None = None, bypass_index=False, query_timeout_mins: int | None = None):
         raise Exception("Azure doesn't support exec_lqp")
 
-    def exec_raw(self, database:str, engine:str|None, raw_code:str, readonly=True, *, inputs: Dict | None = None, nowait_durable=False, headers: Dict | None = None, raw_results=True):
+    def exec_raw(self, database:str, engine:str|None, raw_code:str, readonly=True, *, inputs: Dict | None = None, nowait_durable=False, headers: Dict | None = None, raw_results=True, query_timeout_mins: int | None = None):
+        if query_timeout_mins is not None or self.config.get("query_timeout_mins", None) is not None:
+            config_file_path = getattr(self.config, 'file_path', None)
+            raise AzureUnsupportedQueryTimeoutException(config_file_path=config_file_path)
         if engine is None:
             engine = self.get_default_engine_name()
         try:
@@ -289,7 +292,7 @@ class Resources(ResourcesBase):
     # Exec format
     #--------------------------------------------------
 
-    def exec_format(self, database: str, engine: str, raw_code: str, task:m.Task, format:str, inputs: Dict | None = None, readonly: bool = True, nowait_durable=False, skip_invalid_data=False, headers: Dict | None = None) -> Any: # @FIXME: Better type annotation
+    def exec_format(self, database: str, engine: str, raw_code: str, cols:List[str], format:str, inputs: Dict | None = None, readonly: bool = True, nowait_durable=False, skip_invalid_data=False, headers: Dict | None = None, query_timeout_mins: int | None = None) -> Any: # @FIXME: Better type annotation
         raise Exception("Azure doesn't support alternative formats yet")
 
     def to_model_type(self, model:dsl.Graph, name: str, source:str):
