@@ -28,6 +28,10 @@ PUBLIC_CONFIG_KEYS = [
     "ensure_change_tracking",
     "download_url_type",
     "use_direct_access",
+    # query_timeout_mins allows to specify a timeout in minutes applied to all queries. When
+    # a query execution time exceeds this timeout, the query will be aborted. This is useful
+    # to avoid long-running queries that can incur high costs.
+    "query_timeout_mins",
 ]
 
 #--------------------------------------------------
@@ -464,6 +468,13 @@ class Config():
             if not self.file_path:
                 self.file_path = "__inline__"
         self._handle_snowflake_fallback_configurations()
+        # Check if Azure platform is being used without the legacy dependency
+        if self.get("platform", "") == "azure":
+            try:
+                import railib # noqa
+            except ImportError:
+                from relationalai.errors import AzureLegacyDependencyMissingException
+                raise AzureLegacyDependencyMissingException() from None
 
     def fetch(self, profile:str|None=None):
         from relationalai.environments import runtime_env, TerminalEnvironment
