@@ -61,6 +61,12 @@ class Int128Array(ExtensionArray):
     def _from_factorized(cls, values, original):
         return Int128Array(values)
 
+    @classmethod
+    def _concat_same_type(cls, to_concat):
+        # to_concat is a sequence of your arrays
+        data = np.concatenate([x._data for x in to_concat])
+        return cls(data)
+
     def __getitem__(self, item):
         if isinstance(item, int):
             return self._data[item]
@@ -218,7 +224,6 @@ def merge_columns(left:DataFrame, right:DataFrame, on:List[str]):
     if not len(left.columns):
         for col in on:
             left.insert(loc=len(left.columns), column=col, value=np.nan)
-    if left.empty:
         col_count = len(left.columns)
         left.insert(loc=col_count, column=f'new_column{col_count}', value=np.nan)
     if right.empty:
