@@ -22,6 +22,7 @@ import sys
 from pandas import DataFrame
 
 from relationalai.environments import runtime_env, SnowbookEnvironment
+from relationalai.tools.constants import QUERY_ATTRIBUTES_HEADER
 
 from .clients.client import Client
 
@@ -34,9 +35,7 @@ from .errors import FilterAsValue, Errors, InvalidPropertySetException, Multiple
 #--------------------------------------------------
 
 RESERVED_PROPS = ["add", "set", "persist", "unpersist"]
-
 MAX_QUERY_ATTRIBUTE_LENGTH = 255
-QUERY_ATTRIBUTES_HEADER = "X-Query-Attributes"
 
 Value = Union[
     "Expression",
@@ -1736,9 +1735,9 @@ class Graph:
                         file_path = os.path.join(root, file)
                         self._client.load_raw_file(file_path)
 
-    def exec_raw(self, code:str, readonly=False, raw_results=True, abort_on_error=True, inputs={}):
+    def exec_raw(self, code:str, readonly=False, raw_results=True, abort_on_error=True, inputs={}, query_timeout_mins: Optional[int]=None):
         try:
-            return self._client.exec_raw(code, readonly=readonly, raw_results=raw_results, inputs=inputs, abort_on_error=abort_on_error)
+            return self._client.exec_raw(code, readonly=readonly, raw_results=raw_results, inputs=inputs, abort_on_error=abort_on_error, query_timeout_mins=query_timeout_mins)
         except KeyboardInterrupt as e:
             print("Canceling transactions...")
             self.resources.cancel_pending_transactions()
