@@ -1,6 +1,6 @@
 import warnings
 
-from relationalai.semantics.std import range, hash, cast, dates, math, strings, decimals, integers, pragmas, \
+from relationalai.semantics.std import range, hash, cast, datetime as dates, math, strings, decimals, integers, pragmas, \
     constraints, uuid_to_string
 
 __all__ = [
