@@ -1,4 +1,3 @@
-import warnings
 from collections import defaultdict
 from typing import Optional
 from relationalai import Config
@@ -8,8 +7,8 @@ from relationalai.early_access.dsl.adapters.orm.parser import ORMParser
 from relationalai.early_access.dsl.core.utils import to_pascal_case
 from relationalai.early_access.dsl.orm.constraints import RingType, ValueComparisonType, Range
 from relationalai.early_access.dsl.orm.models import Model
-from relationalai.early_access.builder import Concept, Integer, String, DateTime, Decimal, Date, Float
-from relationalai.early_access.metamodel.util import NameCache
+from relationalai.semantics import Concept, Integer, String, DateTime, Decimal, Date, Float
+from relationalai.semantics.metamodel.util import NameCache
 from datetime import date, datetime
 from decimal import Decimal as PyDecimal
 
@@ -43,7 +42,7 @@ class ORMAdapterQB:
         Concept.builtins["Bool"]: bool,
         Concept.builtins["Date"]: date,
         Concept.builtins["DateTime"]: datetime,
-        Concept.builtins["Decimal128"]: PyDecimal
+        Concept.builtins["Decimal"]: PyDecimal
     }
 
     _ring_type_mapping = {
@@ -180,7 +179,6 @@ class ORMAdapterQB:
             player_entity = model.lookup_concept(player)
             reading = self._build_reading(model, rdo)
             relationship = model.Relationship(reading)
-            self._set_role_verbalization(relationship, rdo)
             short_name = self._pick_short_name(model, player_entity, fact_type, relationship._readings[0])
             cached_name = self.name_cache.get_name(relationship._readings[0], f"{player_entity._name}.{short_name}")
             short_name = cached_name.split(".")[1]
@@ -204,10 +202,7 @@ class ORMAdapterQB:
                 if role_id in role_cardinality_constraints.keys():
                     model.role_cardinality(role, self._build_constraint_ranges(role_cardinality_constraints[role_id], int))
 
-
             # Adding alternative readings
-            self._validate_role_verbalization(reading_orders)
-
             if len(reading_orders) > 1:
                 for rdo in reading_orders[1:]:
                     other_player = object_types[rdo.roles[0].role.player].name
@@ -335,10 +330,14 @@ class ORMAdapterQB:
         if reading_order.front_text is not None:
             rel_args.append(f"{reading_order.front_text} ")
         for rdo_role in reading_order.roles:
+            if rdo_role.prefix:
+                rel_args.append(f"{rdo_role.prefix}- ")
             p = model.lookup_concept(object_types[rdo_role.role.player].name)
-            rel_args.append(f"{{{p}}}") if rdo_role.role.name == "" else rel_args.append(f"{{{rdo_role.role.name}:{p}}}")
+            rel_args.append(f"{{{p}}} ") if rdo_role.role.name == "" else rel_args.append(f"{{{rdo_role.role.name}:{p}}} ")
+            if rdo_role.postfix:
+                rel_args.append(f"-{rdo_role.postfix} ")
             if rdo_role.text is not None:
-                rel_args.append(f" {rdo_role.text} ")
+                rel_args.append(f"{rdo_role.text} ")
         return ''.join(rel_args).strip()
 
     def _rename_to_ref_mode(self, model, first_player_entity, fact_type):
@@ -426,30 +425,3 @@ class ORMAdapterQB:
             else:
                 constraint_ranges.append(tp(rg))
         return constraint_ranges
-
-    @staticmethod
-    def _set_role_verbalization(relationship, rdo):
-        for idx, ro in enumerate(rdo.roles):
-            qb_role = relationship._roles()[idx]
-            text_chunks = [None, None]
-            if ro.prefix:
-                text_chunks[0] = ro.prefix
-            if ro.postfix:
-                text_chunks[1] = ro.postfix
-            if text_chunks[0] or text_chunks[1]:
-                qb_role.verbalization(*text_chunks)
-
-    @staticmethod
-    def _validate_role_verbalization(reading_orders):
-        if len(reading_orders) > 1:
-            relationship = reading_orders[0].roles[0].role.relationship_name
-            for ro in reading_orders[0].roles:
-                prefix = ro.prefix
-                postfix = ro.postfix
-                player = ro.role.player
-                #  A role player must have the same prefix in all readings
-                if not all(alt_ro.prefix == prefix for rdo in reading_orders[1:] for alt_ro in rdo.roles if alt_ro.role.player == player):
-                    warnings.warn(f"Ignoring incompatible prefix in alternative readings of {relationship}.")
-                #  A role player must have the same postfix in all readings
-                if not all(alt_ro.postfix == postfix for rdo in reading_orders[1:] for alt_ro in rdo.roles if alt_ro.role.player == player):
-                    warnings.warn(f"Ignoring incompatible postfix in alternative readings of {relationship}")
