@@ -1,5 +1,5 @@
 from io import StringIO
-from typing import Optional
+from typing import Optional, Hashable
 
 import numpy as np
 import pandas as pd
@@ -76,7 +76,7 @@ class BindableCsvColumn(BindableColumn, b.Relationship):
 
 
 class CsvTable(AbstractBindableTable[BindableCsvColumn]):
-    _basic_type_schema: dict[str, str]
+    _basic_type_schema: dict[Hashable, str]
     _csv_data: list[pd.DataFrame]
     _num_rows: int
 
@@ -157,13 +157,13 @@ class CsvSourceModule:
     def _row_to_date_value_rule(column, row, value):
         parse_date = b.Relationship.builtins['parse_date']
         rez = b.Date.ref()
-        where(parse_date(value, 'Y-m-d', rez)).define(column(row, rez))
+        where(parse_date(value, 'yyyy-mm-dd', rez)).define(column(row, rez))
 
     @staticmethod
     def _row_to_date_time_value_rule(column, row, value):
         parse_datetime = b.Relationship.builtins['parse_datetime']
         rez = b.DateTime.ref()
-        where(parse_datetime(value, 'Y-m-d HH:MM:SS z', rez)).define(column(row, rez))
+        where(parse_datetime(value, 'yyy-mm-dd HH:MM:SS z', rez)).define(column(row, rez))
 
     @staticmethod
     def _row_to_decimal_value_rule(column, row, value):
