@@ -4,9 +4,9 @@ from relationalai.early_access.dsl.bindings.common import Binding, IdentifierCon
     SubtypeConceptBinding, BindableTable, BindableAttribute, ReferentConceptBinding
 from relationalai.early_access.dsl.codegen.common import BoundIdentifierConstraint, BoundRelationship
 from relationalai.early_access.dsl.orm.relationships import Role, Relationship
-from relationalai.early_access.metamodel.util import OrderedSet
+from relationalai.semantics.metamodel.util import OrderedSet
 
-import relationalai.early_access.builder as qb
+import relationalai.semantics as qb
 
 class Binder:
     def __init__(self, model):
