@@ -103,8 +103,7 @@ class Weaver:
             # binding for each supertype with a reference scheme.
             #=
             constructor_binding = (
-                self._binder.lookup_constructor_binding(concept, binding.column)
-                if constructed_concept is concept
+                binding if constructed_concept is concept
                 else self._binder.lookup_constructor_binding_by_source(constructed_concept, binding.column.table)
             )
 
