@@ -12,9 +12,9 @@ from relationalai.clients import result_helpers
 from relationalai.clients.snowflake import APP_NAME
 from relationalai.early_access.dsl.ir.compiler import Compiler
 from relationalai.early_access.dsl.ontologies.models import Model
-from relationalai.early_access.metamodel import ir
-from relationalai.early_access.metamodel.visitor import collect_by_type
-from relationalai.early_access.rel.compiler import ModelToRel
+from relationalai.semantics.metamodel import ir
+from relationalai.semantics.metamodel.visitor import collect_by_type
+from relationalai.semantics.rel.compiler import ModelToRel
 from relationalai.clients.config import Config
 from relationalai.tools.constants import USE_GRAPH_INDEX, Generation
 
