@@ -1,6 +1,6 @@
 from relationalai.early_access.dsl import Namespace
 from relationalai.early_access.dsl.core.types import Type
-from relationalai.early_access.metamodel.util import OrderedSet
+from relationalai.semantics.metamodel.util import OrderedSet
 from relationalai.early_access.dsl.core.relations import ExportRelation, RelationSignature
 
 
