@@ -2,9 +2,9 @@ from abc import ABC, abstractmethod
 from typing import Optional
 
 from relationalai.early_access.dsl.orm.measures.initializer import get_reasoner
-from relationalai.early_access.metamodel.util import OrderedSet, ordered_set
+from relationalai.semantics.metamodel.util import OrderedSet, ordered_set
 from relationalai.early_access.dsl.orm.models import Role
-from relationalai.early_access.builder import Concept
+from relationalai.semantics import Concept
 
 from relationalai.early_access.dsl.orm.measures.role_exprs import RoleExpr, SimpleRoleExpr
 
