@@ -3,7 +3,7 @@ from typing import Union, Optional
 
 from relationalai.early_access.dsl.orm.measures.initializer import get_reasoner
 from relationalai.early_access.dsl.orm.models import Role
-from relationalai.early_access.metamodel.util import OrderedSet, ordered_set
+from relationalai.semantics.metamodel.util import OrderedSet, ordered_set
 
 class RoleExpr(ABC):
 
