@@ -8,8 +8,8 @@ from relationalai.early_access.dsl.orm.constraints import Unique, Mandatory, Rol
 from relationalai.early_access.dsl.orm.models import Model
 from relationalai.early_access.dsl.orm.relationships import Role
 from relationalai.early_access.dsl.orm.types import Concept
-from relationalai.early_access.metamodel.util import Printer as BasePrinter, OrderedSet
-import relationalai.early_access.builder as qb
+from relationalai.semantics.metamodel.util import Printer as BasePrinter, OrderedSet
+import relationalai.semantics as qb
 
 MODEL_VAR_NAME = "model"
 
@@ -55,7 +55,8 @@ class Printer(BasePrinter):
     def to_string(self, model: Model, enums_enabled: bool = False, pyi_enabled: bool = False) -> None:
         self._process_constraints(model)
 
-        self._print_nl("import relationalai.early_access.builder as qb")
+        self._print_nl("import relationalai.semantics.internal as qb")
+        self._print_nl("import relationalai.semantics.internal.internal as builder")
         self._print_nl("from relationalai.early_access.dsl.orm.models import Model")
         if pyi_enabled:
             self._print_nl("from relationalai.early_access.dsl.orm.models import Concept")
@@ -65,7 +66,7 @@ class Printer(BasePrinter):
         self._nl()
 
         self._print_nl(f"{MODEL_VAR_NAME} = Model(name='{model.name}', is_primary={model.is_primary}"
-                       f"{self._print_if_true('use_lqp', model.qb_model()._use_lqp)}"
+                       f"{self._print_if_false('use_lqp', model.qb_model()._use_lqp)}"
                        f"{self._print_if_true('use_sql', model.qb_model()._use_sql)}"
                        f"{self._print_if_true('wide_outputs', model.qb_model()._wide_outputs)}"
                        f"{self._print_if_true('dry_run', model.qb_model()._dry_run)})")
@@ -200,23 +201,12 @@ class Printer(BasePrinter):
             # print a root Relationship
             self._print_nl(f"{self._get_relationship_name(rel)} = "
                            f"Relationship('{rel._madlib}'{self._print_if_not_empty_and_not_equal('short_name', rel._passed_short_name, rel._name)})")
-            self._handle_role_verbalization(rel)
 
             # print remaining RelationshipReadings if any
             for r in rel._readings[1:]:
                 self._print_nl(f"{self._get_relationship_name(r)} = "
                                f"{self._get_relationship_name(rel)}.alt('{r._madlib}'{self._print_if_not_empty_and_not_equal('short_name', r._passed_short_name, r._name)})")
 
-    def _handle_role_verbalization(self, rel):
-        for ro in rel._roles():
-            verbalization_data = []
-            if ro.prefix:
-                verbalization_data.append(f'prefix="{ro.prefix}"')
-            if ro.postfix:
-                verbalization_data.append(f'postfix="{ro.postfix}"')
-            if verbalization_data:
-                self._print_nl(f"{self._get_role_name(ro)}.verbalization({', '.join(verbalization_data)})")
-
     def _handle_ref_schemes(self, model: Model) -> None:
         for concept_name, concept in model.entity_types_map().items():
             if not isinstance(concept, Concept) or not concept._reference_schemes:
@@ -229,9 +219,7 @@ class Printer(BasePrinter):
     def _handle_uniqueness_constraints(self) -> None:
         for constraint in self._unique_constraints:
             if not constraint.is_preferred_identifier:
-                elements = []
-                for role in constraint.roles():
-                    elements.append(self._get_role_name(role))
+                elements = [self._get_role_name(role) for role in constraint.roles()]
                 self._print_nl(f"Unique({', '.join(elements)})")
 
     def _handle_mandatory_constraints(self):
@@ -262,19 +250,13 @@ class Printer(BasePrinter):
 
     def _handle_ring_constraints(self):
         for constraint in self._ring_constraints:
-            elements = []
-            for role in constraint.roles():
-                elements.append(self._get_role_name(role))
-            cst_types = []
-            for tp in constraint.types:
-                cst_types.append(f"{tp}")
+            elements = [self._get_role_name(role) for role in constraint.roles()]
+            cst_types = [f"{tp}" for tp in constraint.types]
             self._print_nl(f"RingConstraint([{', '.join(cst_types)}], {', '.join(elements)})")
 
     def _handle_value_comparison_constraints(self):
         for constraint in self._value_comparison_constraints:
-            elements = []
-            for role in constraint.roles():
-                elements.append(self._get_role_name(role))
+            elements = [self._get_role_name(role) for role in constraint.roles()]
             self._print_nl(f"ValueComparisonConstraint({constraint.type}, {', '.join(elements)})")
 
     def _handle_role_subset_constraints(self):
@@ -362,7 +344,10 @@ class Printer(BasePrinter):
 
     def _get_type(self, t: Concept) -> str:
         name = _get_concept_name(t)
-        return f"qb.{name}" if name in Concept.builtins else name
+        if name.startswith("Decimal"):
+            return f"builder.decimal_concept_by_name('{name}')"
+        else:
+            return f"qb.{name}" if name in Concept.builtins else name
 
     @staticmethod
     def _get_constraint_values(constraint):
@@ -394,6 +379,10 @@ class Printer(BasePrinter):
     def _print_if_true(label: str, value: bool) -> str:
         return f", {label}=True" if value else ""
 
+    @staticmethod
+    def _print_if_false(label: str, value: bool) -> str:
+        return f", {label}=False" if not value else ""
+
     @staticmethod
     def _is_complex_role_sequence(role_sequences):
         return not all(len(item) == 1 for item in role_sequences)
@@ -412,7 +401,8 @@ class InterfacePrinter(BasePrinter):
         if enums_enabled:
             self._print_nl("from enum import Enum")
         self._print_nl("from typing import Union")
-        self._print_nl("import relationalai.early_access.builder as qb")
+        self._print_nl("import relationalai.semantics.internal as qb")
+        self._print_nl("import relationalai.semantics.internal.internal as builder")
         self._print_nl("from relationalai.early_access.dsl.orm.models import Concept, Model")
         self._nl()
         self._print_nl(f"{MODEL_VAR_NAME}: Model")
