@@ -1,14 +1,31 @@
-import logging
-from typing import Optional
+from enum import Enum
+from typing import Optional, Union
 
-import relationalai.early_access.builder as qb
+import relationalai.semantics as qb
 from relationalai.early_access.dsl.orm.constraints import Constraint, Unique, Mandatory, RoleValueConstraint, \
-    InclusiveSubtypeConstraint, ExclusiveSubtypeConstraint
+    InclusiveSubtypeConstraint, ExclusiveSubtypeConstraint, InclusiveRoleConstraint, ExclusiveRoleConstraint, \
+    RingConstraint, RoleCardinalityConstraint, ValueConstraint, CardinalityConstraint, FrequencyConstraint, \
+    ValueComparisonConstraint, RoleSubsetConstraint, EqualityConstraint
 from relationalai.early_access.dsl.orm.relationships import Role, Relationship
-from relationalai.early_access.metamodel.util import OrderedSet
-from relationalai.early_access.dsl.core import warn
+from relationalai.semantics.metamodel.util import OrderedSet, ordered_set
 
 
+class Multiplicity(Enum):
+    AT_MOST_ONE = 0
+    EXACTLY_ONE = 1
+    ZERO_OR_MORE = 2
+
+    def many(self) -> bool:
+        return self == Multiplicity.ZERO_OR_MORE
+
+    def qualifier(self) -> str:
+        if self == Multiplicity.AT_MOST_ONE:
+            return "at most one"
+        elif self == Multiplicity.EXACTLY_ONE:
+            return "exactly one"
+        else:
+            return "more than one"
+
 class OntologyReasoner:
     def __init__(self):
 
@@ -29,6 +46,16 @@ class OntologyReasoner:
         self._role_value_constraints: OrderedSet[RoleValueConstraint] = OrderedSet()
         self._inclusive_subtype_constraints: OrderedSet[InclusiveSubtypeConstraint] = OrderedSet()
         self._exclusive_subtype_constraints: OrderedSet[ExclusiveSubtypeConstraint] = OrderedSet()
+        self._inclusive_role_constraints: OrderedSet[InclusiveRoleConstraint] = OrderedSet()
+        self._exclusive_role_constraints: OrderedSet[ExclusiveRoleConstraint] = OrderedSet()
+        self._ring_constraints: OrderedSet[RingConstraint] = OrderedSet()
+        self._value_comparison_constraints: OrderedSet[ValueComparisonConstraint] = OrderedSet()
+        self._role_subset_constraints: OrderedSet[RoleSubsetConstraint] = OrderedSet()
+        self._equality_constraints: OrderedSet[EqualityConstraint] = OrderedSet()
+        self._frequency_constraints: OrderedSet[FrequencyConstraint] = OrderedSet()
+        self._cardinality_constraints: OrderedSet[CardinalityConstraint] = OrderedSet()
+        self._role_cardinality_constraints: OrderedSet[RoleCardinalityConstraint] = OrderedSet()
+        self._value_constraints: OrderedSet[ValueConstraint] = OrderedSet()
 
         self._subtype_map: dict[qb.Concept, OrderedSet[qb.Concept]] = {}
         self._supertype_map: dict[qb.Concept, OrderedSet[qb.Concept]] = {}
@@ -68,8 +95,26 @@ class OntologyReasoner:
             self._inclusive_subtype_constraints.add(constraint)
         elif isinstance(constraint, ExclusiveSubtypeConstraint):
             self._exclusive_subtype_constraints.add(constraint)
-        else:
-            logging.warning(f"Unknown constraint type: {type(constraint)}")
+        elif isinstance(constraint, InclusiveRoleConstraint):
+            self._inclusive_role_constraints.add(constraint)
+        elif isinstance(constraint, ExclusiveRoleConstraint):
+            self._exclusive_role_constraints.add(constraint)
+        elif isinstance(constraint, RingConstraint):
+            self._ring_constraints.add(constraint)
+        elif isinstance(constraint, ValueComparisonConstraint):
+            self._value_comparison_constraints.add(constraint)
+        elif isinstance(constraint, RoleSubsetConstraint):
+            self._role_subset_constraints.add(constraint)
+        elif isinstance(constraint, EqualityConstraint):
+            self._equality_constraints.add(constraint)
+        elif isinstance(constraint, FrequencyConstraint):
+            self._frequency_constraints.add(constraint)
+        elif isinstance(constraint, CardinalityConstraint):
+            self._cardinality_constraints.add(constraint)
+        elif isinstance(constraint, RoleCardinalityConstraint):
+            self._role_cardinality_constraints.add(constraint)
+        elif isinstance(constraint, ValueConstraint):
+            self._value_constraints.add(constraint)
 
     #=
     # Constraints: uniqueness and reference schemes.
@@ -243,6 +288,12 @@ class OntologyReasoner:
                 return supertype
         return None
 
+    def in_subtype_closure(self, sub: qb.Concept, sup: qb.Concept) -> bool:
+        """
+        Returns true if sub is in the subtype closure of sup.
+        """
+        return sub in self._subtype_closure.get(sup, OrderedSet())
+
     def is_mandatory_role(self, role: Role) -> bool:
         """
         Returns true if the role is a mandatory role.
@@ -256,19 +307,71 @@ class OntologyReasoner:
         """
         return not any(c in self._internal_unique_constraints for c in self._role_unique_constraints.get(role, OrderedSet()))
 
-    # [REKS: TODO] Need to loosen this requirement so that we admit
-    #              compatible roles when the player of r1 is a subtype
-    #              of that of r2 or vice versa.
     def are_roles_compatible(self, role1: Role, role2: Role) -> bool:
-        warn(f"Checking compatibility of roles played by {str(role1.player())} and {str(role2.player())} without reasoning about subtypes")
-        return role1.player() == role2.player()
+        """
+        Returns true if the two roles are compatible, i.e. they have the same player or
+        one is a subtype of the other.
+        """
+        return (
+            role1.player() is role2.player() or
+            self.in_subtype_closure(role1.player(), role2.player()) or
+            self.in_subtype_closure(role2.player(), role1.player())
+        )
 
-    # [REKS: TODO] Need to do the subtype/supertype reasoning required
-    #              to properly answer this w/o requring c1 to be equal
-    #              to c2
+    # [REKS: TODO] Need to improve to handle multiple inheritance properly
     def least_supertype(self, concept1: qb.Concept, concept2: qb.Concept) -> Optional[qb.Concept]:
-        warn(f"Trying to find least supertype of {str(concept1)} and {str(concept2)} without reasoning about subtypes")
-        return None if str(concept1) != str(concept2) else concept1
+        """
+        Returns the least common supertype of the two concepts, if it exists.
+        """
+        supertypes1 = ordered_set(concept1, *self._supertype_closure.get(concept1, OrderedSet()))
+        supertypes2 = ordered_set(concept2, *self._supertype_closure.get(concept2, OrderedSet()))
+        common = supertypes1 & supertypes2
+        return common.first() if common else None
+
+    def internal_ucs_of(self, relationship: Union[qb.Relationship, qb.RelationshipReading]) -> OrderedSet[Unique]:
+        """
+        Returns the set of internal uniqueness constraints that span any role that
+        is part of relationship
+        """
+        oset = OrderedSet()
+        ucdict = self._role_unique_constraints
+        for role in relationship._roles():
+            if role in ucdict:
+                for uc in ucdict[role]:
+                    if uc in self._internal_unique_constraints:
+                        oset.add(uc)
+        return oset
+
+
+    # Let *role* be a Role of some Relationship R. The *point multiplicity* of
+    # role considers maximal subsets F of facts of R in which, for every Role r
+    # where:
+    #
+    #     r in roles(R) \ {role}
+    #
+    # there exists some object o that plays r in each fact in F. That is, each
+    # F is some set of R facts in which every Role but *role* is bound to the
+    # same object in each fact in F.
+    #
+    # The point multiplicity of *role* is then either:
+    #   - AT_MOST_ONE, if for every F, |F| <= 0
+    #   - EXACTLY_ONE, if for every F, |F| == 1
+    #   - ZERO_OR_MORE, otherwise
+    #
+    def point_multiplicity(self, role: Role) -> Multiplicity:
+        relationship = role._part_of()
+        ucs = self.internal_ucs_of(relationship)
+        if len(ucs) > 0:
+            # If there exists a UC that does NOT span role, then
+            # that UC determines the multiplicity of role.
+                for uc in ucs:
+                    if role not in uc.roles():
+                        # Then multiplicity is either AT_MOST_ONE or ONE
+                        sibling_role = role.sibling()
+                        if sibling_role is not None and self.is_mandatory_role(sibling_role):
+                            return Multiplicity.EXACTLY_ONE
+                        return Multiplicity.AT_MOST_ONE
+        return Multiplicity.ZERO_OR_MORE
 
     @staticmethod
     def _extract_unique_role_from_uc(uc: Unique) -> Role:
