@@ -1,7 +1,6 @@
 from __future__ import annotations
 
 import keyword
-import re
 from relationalai.early_access.dsl.core.utils import camel_to_snake, to_rai_way_string
 import relationalai.early_access.dsl.orm.relationships as orm_qb
 
@@ -10,7 +9,6 @@ import relationalai.early_access.dsl.orm.relationships as orm_qb
 # to the rai way conventions.
 def generate_rai_way_name(reading: orm_qb.RelationshipReading):
     # These two dict and the roles array are indexed in same way, using the role position in the madlib
-    following_texts = extract_roles_data_from_madlib(reading)
     roles = reading._roles()
 
     # A role name must be different from player and player_i (used for multiple occurrences of the same player)
@@ -23,28 +21,34 @@ def generate_rai_way_name(reading: orm_qb.RelationshipReading):
 
     # We invoke this method with an index to compose together the textual parts of a single role (prefix, role/player
     # name, postfix and following text.
-    def compose_role_textual_parts(idx: int, drop_is_has=True):
+    def compose_role_textual_parts(idx: int, drop_is_has=True, drop_role_name=False):
         textual_parts = []
         role = roles[idx]
-        if role.prefix:
-            textual_parts.append(camel_to_snake(role.prefix))
+        role_occurrence = reading._role_in_reading[idx]
+        role_name = get_role_name(idx)
+        if role_occurrence._prefix and (not role_name or drop_role_name):
+            textual_parts.append(camel_to_snake(role_occurrence._prefix))
         if idx > 0:
-            role_name = get_role_name(idx)
-            textual_parts.append(camel_to_snake(role_name) if role_name else camel_to_snake(role.player()._name))
-        if role.postfix:
-            textual_parts.append(camel_to_snake(role.postfix))
-        if following_texts[idx]:
-            rai_way_string = to_rai_way_string(following_texts[idx], drop_is_has)
+            textual_parts.append(camel_to_snake(role_name) if role_name and not drop_role_name
+                                 else camel_to_snake(role.player()._name))
+        if role_occurrence._postfix and (not role_name or drop_role_name):
+            textual_parts.append(camel_to_snake(role_occurrence._postfix))
+        follows = reading._follows[idx]
+        if follows:
+            rai_way_string = to_rai_way_string(follows, drop_is_has)
             if rai_way_string:
                 textual_parts.append(rai_way_string)
         return '_'.join(textual_parts).replace(" ", "_")
 
     # Generate the rai way name for the binary case, when most of the role data in not present.
     def generate_simple_binary_name():
-        simple_name = to_rai_way_string(following_texts[0])
-        if (simple_name and not following_texts[1] and not keyword.iskeyword(simple_name) and
-                not roles[0].prefix and not roles[0].postfix and not roles[1].prefix and not roles[1].postfix):
-            return simple_name
+        follows = reading._follows[0]
+        if follows:
+            simple_name = to_rai_way_string(follows)
+            if (simple_name and not reading._follows[1] and not keyword.iskeyword(simple_name) and
+                not reading._role_in_reading[0]._prefix and not reading._role_in_reading[0]._postfix and
+                not reading._role_in_reading[1]._prefix and not reading._role_in_reading[1]._postfix):
+                return simple_name
         return None
 
     # Unary case
@@ -69,23 +73,7 @@ def generate_rai_way_name(reading: orm_qb.RelationshipReading):
     else:
         all_roles_components = []
         for i, role in enumerate(roles):
-            role_component = compose_role_textual_parts(i,False)
+            role_component = compose_role_textual_parts(i,False, True)
             all_roles_components.append(role_component)
         rel_name = "_".join(all_roles_components)
         return rel_name
-
-
-# Returns the dictionary of texts following a role (empty string if not present) indexed by the role position
-def extract_roles_data_from_madlib(reading: orm_qb.RelationshipReading):
-    # This pattern capture the player name (with optional role name) and the following text for each
-    # role in the madlib string
-    # \{([^{}]*)\}   Player or role_name:Player
-    # ([^{]*)       Following text
-    pattern = r"\{([^{}]*)\}([^{]*)"
-
-    matches = re.findall(pattern, reading._madlib)
-    following_texts = dict()
-
-    for i, (_, follows) in enumerate(matches):
-        following_texts[i] = follows.strip()
-    return following_texts
