@@ -1,7 +1,7 @@
 from relationalai.early_access.dsl.core import warn
 from relationalai.early_access.dsl.core.constraints import FalseConstraint
 from relationalai.early_access.dsl.core.constraints.predicate.atomic import reduce_relational_atoms
-from relationalai.early_access.dsl.core.constraints.scalar import entails, fold_constants, reduce_constraints, unsatisfiable
+from relationalai.early_access.dsl.core.constraints.scalar import fold_constants
 from relationalai.early_access.dsl.core.exprs.scalar import Literal, box_number
 from relationalai.early_access.dsl.core.logic import LogicFragment
 from relationalai.early_access.dsl.core.logic.helper import occurrences, instantiate_type_constraints_for
@@ -56,23 +56,6 @@ class SchemaFragment(LogicFragment):
     #
     def assume_precondition(self, pre):
         tconsts = self._sconstraints
-
-        tlen = len(tconsts) + 1
-        while(len(tconsts) < tlen):
-            tlen = len(tconsts)
-
-            for c in tconsts:
-                # We can remove c from self if c is entailed by:
-                #
-                #    context =^= (self \ {c}) u pre
-                #
-                tcprime = { k: tconsts[k] for k in tconsts if k != c }
-                context = tcprime | pre._sconstraints
-
-                if entails(context, tconsts[c]):
-                    tconsts = tcprime
-                    break
-
         new_trans = self.duplicate()
         new_trans._sconstraints = tconsts
         return new_trans
@@ -334,39 +317,7 @@ class SchemaFragment(LogicFragment):
         return len(self._relations) == 0
 
     def reduce(self):
-        s = self.reduce_relational_atoms()
-        if s is None:
-            return s
-        else:
-            return s.reduce_scalar_constraints()
-
-    # Simplify the scalar propositional constraints of this Schema using
-    # satisfiability checks over numeric inequalities and expressions.
-    #
-    # If we can decide a Schema is unstaisfiable, return None
-    #
-    def reduce_scalar_constraints(self):
-        C = self._sconstraints.copy()
-
-        atoms = self._atoms
-        valueTypeConstraints = infer_referent_constraints(atoms)
-        for v in valueTypeConstraints:
-            C[v] = valueTypeConstraints[v]
-
-        if unsatisfiable(C):
-            return None
-
-        reduced = reduce_constraints(C)
-        newprops = {c: reduced[c] for c in reduced if c not in valueTypeConstraints}
-
-        s = SchemaFragment()
-        s._scalars = self._scalars
-        s._relations = self._relations
-        s._universals = self._universals
-        s._atoms = self._atoms
-        s._sconstraints = newprops
-
-        return s
+        return self.reduce_relational_atoms()
 
     # Simplify thus Schema by projecting out decls that are equated with other
     # decls via predicate constraints. These commonly arise when decorating
