@@ -1,9 +1,8 @@
 import dataclasses
 from typing import Optional
 
-from relationalai.early_access.builder import (Float, Int128, String, Bool, Date, DateTime,
-                                               Concept as QBConcept, Decimal64, Decimal128)
-
+from relationalai.semantics.internal import internal as builder
+from relationalai.semantics import Float, Int128, String, Bool, Date, DateTime, Concept as QBConcept
 
 #=
 # Physical metadata for a Snowflake table.
@@ -56,6 +55,7 @@ _sf_type_mapping = {
     'date': Date,
     'datetime': DateTime,
     'timestamp_ntz': DateTime,
+    'timestamp_tz': DateTime,
     'boolean': Bool,
     'float': Float,
 }
@@ -73,9 +73,4 @@ def _map_rai_type(col: ColumnMetadata) -> QBConcept:
 def _map_decimal_type(precision: Optional[int], scale: Optional[int]) -> QBConcept:
     if precision is None or scale is None:
        raise ValueError("Size and scale must be provided for decimal type mapping")
-    if 1 <= precision <= 18:
-        return Decimal64
-    elif 18 < precision <= 38:
-        return Decimal128
-    else:
-        raise ValueError(f"Unsupported precision {precision} for decimal type mapping")
+    return builder.decimal_concept(precision, scale)
