@@ -2395,3 +2395,61 @@ class UnsupportedColumnTypesWarning(RAIWarning):
 
 {note}
 """)
+
+class QueryTimeoutExceededException(RAIException):
+    def __init__(self, timeout_mins: int, query_id: str | None = None, config_file_path: str | None = None):
+        self.timeout_mins = timeout_mins
+        self.name = "Query Timeout Exceeded"
+        self.message = f"Query execution time exceeded the specified timeout of {self.timeout_mins} minutes."
+        self.query_id = query_id or ""
+        self.config_file_path = config_file_path or ""
+        self.content = self.format_message()
+        super().__init__(self.message, self.name, self.content)
+
+    def format_message(self):
+        return textwrap.dedent(f"""
+        Query execution time exceeded the specified timeout of {self.timeout_mins} minutes{f' for query with ID: {self.query_id}' if self.query_id else ''}.
+
+        Consider increasing the 'query_timeout_mins' parameter in your configuration file{f' (stored in {self.config_file_path})' if self.config_file_path else ''} to allow more time for query execution.
+        """)
+
+
+#--------------------------------------------------
+# Azure Exceptions
+#--------------------------------------------------
+
+class AzureUnsupportedQueryTimeoutException(RAIException):
+    def __init__(self, config_file_path: str | None = None):
+        self.message = "Query timeouts aren't supported on platform Azure."
+        self.name = "Azure Unsupported Query Timeout Error"
+        self.config_file_path = config_file_path or ""
+        self.content = self.format_message()
+        super().__init__(self.message, self.name, self.content)
+
+    def format_message(self):
+        return textwrap.dedent(f"""
+        {self.message}
+
+        Please remove the 'query_timeout_mins' from your configuration file{f' (stored in {self.config_file_path})' if self.config_file_path else ''} when running on platform Azure.
+        """)
+
+class AzureLegacyDependencyMissingException(RAIException):
+    def __init__(self):
+        self.message = "The Azure platform requires the 'legacy' extras to be installed."
+        self.name = "Azure Legacy Dependency Missing"
+        self.content = self.format_message()
+        super().__init__(self.message, self.name, self.content)
+
+    def format_message(self):
+        return textwrap.dedent("""
+        The Azure platform requires the 'rai-sdk' package, which is not installed.
+
+        To use the Azure platform, please install the legacy extras:
+
+            pip install relationalai[legacy]
+
+        Or if upgrading an existing installation:
+
+            pip install --upgrade relationalai[legacy]
+        """)
+
