@@ -4,7 +4,7 @@ import atexit
 import re
 
 from pandas import DataFrame
-from typing import Any, Optional
+from typing import Any, Optional, Literal
 from snowflake.snowpark import Session
 import relationalai as rai
 
@@ -12,6 +12,7 @@ from relationalai import debugging
 from relationalai.semantics.lqp import result_helpers
 from relationalai.semantics.metamodel import ir, factory as f, executor as e
 from relationalai.semantics.lqp.compiler import Compiler
+from relationalai.semantics.lqp.intrinsics import mk_intrinsic_datetime_now
 from relationalai.semantics.lqp.types import lqp_type_to_sql
 from lqp import print as lqp_print, ir as lqp_ir
 from lqp.parser import construct_configure
@@ -20,8 +21,8 @@ from relationalai.clients.config import Config
 from relationalai.clients.snowflake import APP_NAME
 from relationalai.clients.types import TransactionAsyncResponse
 from relationalai.clients.util import IdentityParser
-from relationalai.tools.constants import USE_DIRECT_ACCESS
-
+from relationalai.tools.constants import USE_DIRECT_ACCESS, QUERY_ATTRIBUTES_HEADER
+from relationalai.tools.query_utils import prepare_metadata_for_headers
 
 class LQPExecutor(e.Executor):
     """Executes LQP using the RAI client."""
@@ -59,7 +60,7 @@ class LQPExecutor(e.Executor):
                 if not self.dry_run:
                     self.engine = self._resources.get_default_engine_name()
                     if not self.keep_model:
-                        atexit.register(self._resources.delete_graph, self.database, True)
+                        atexit.register(self._resources.delete_graph, self.database, True, "lqp")
         return self._resources
 
     # Checks the graph index and updates it if necessary
@@ -87,7 +88,15 @@ class LQPExecutor(e.Executor):
         assert self.engine is not None
 
         with debugging.span("poll_use_index", sources=sources, model=model, engine=engine_name):
-            resources.poll_use_index(app_name, sources, model, self.engine, engine_size, program_span_id)
+            resources.poll_use_index(
+                app_name=app_name,
+                sources=sources,
+                model=model,
+                engine_name=self.engine,
+                engine_size=engine_size,
+                language="lqp",
+                program_span_id=program_span_id,
+            )
 
     def report_errors(self, problems: list[dict[str, Any]], abort_on_error=True):
         from relationalai import errors
@@ -119,7 +128,17 @@ class LQPExecutor(e.Executor):
                 elif code == "PYREL_ERROR":
                     pyrel_errors[problem["props"]["pyrel_id"]].append(problem)
                 elif abort_on_error:
-                    all_errors.append(errors.RelQueryError(problem, source))
+                    e = errors.RelQueryError(problem, source)
+
+                    if code == 'SYSTEM_INTERNAL':
+                        supplementary_message = "Troubleshooting:\n" + \
+                            "  1. Please retry with a new name for your model. This can work around state-related issues.\n" + \
+                            "  2. If the error persists, please retry with the `use_lqp` flag set to `False`, for example:\n" + \
+                            "       `model = Model(..., use_lqp=False)`\n" + \
+                            "     This will switch the execution to the legacy backend, which may avoid the issue with some performance cost.\n"
+
+                        e.content = f"{e.content}{supplementary_message}"
+                    all_errors.append(e)
             else:
                 if code == "ARITY_MISMATCH":
                     errors.ArityMismatch(problem, source)
@@ -248,11 +267,47 @@ class LQPExecutor(e.Executor):
 
         return ", ".join(fields)
 
+    def _construct_configure(self):
+        config_dict = {}
+        # Only set the IVM flag if there is a value in `config`. Otherwise, let
+        # `construct_configure` set the default value.
+        ivm_flag = self.config.get('reasoner.rule.incremental_maintenance', None)
+        if ivm_flag:
+            config_dict['ivm.maintenance_level'] = lqp_ir.Value(value=ivm_flag, meta=None)
+        return construct_configure(config_dict, None)
+
+    def _compile_intrinsics(self) -> lqp_ir.Epoch:
+        """Construct an epoch that defines a number of built-in definitions used by the
+        emitter."""
+        with debugging.span("compile_intrinsics") as span:
+            debug_info = lqp_ir.DebugInfo(id_to_orig_name={}, meta=None)
+            intrinsics_fragment = lqp_ir.Fragment(
+                id = lqp_ir.FragmentId(id=b"__pyrel_lqp_intrinsics", meta=None),
+                declarations = [
+                    mk_intrinsic_datetime_now(),
+                ],
+                debug_info = debug_info,
+                meta = None,
+            )
+
+            span["compile_type"] = "intrinsics"
+            span["lqp"] = lqp_print.to_string(intrinsics_fragment, {"print_names": True, "print_debug": False, "print_csv_filename": False})
+
+            return lqp_ir.Epoch(
+                writes=[
+                    lqp_ir.Write(write_type=lqp_ir.Define(fragment=intrinsics_fragment, meta=None), meta=None)
+                ],
+                meta=None,
+            )
+
     def compile_lqp(self, model: ir.Model, task: ir.Task):
+        configure = self._construct_configure()
+
         model_txn = None
         if self._last_model != model:
             with debugging.span("compile", metamodel=model) as install_span:
                 _, model_txn = self.compiler.compile(model, {"fragment_id": b"model"})
+                model_txn = txn_with_configure(model_txn, configure)
                 install_span["compile_type"] = "model"
                 install_span["lqp"] = lqp_print.to_string(model_txn, {"print_names": True, "print_debug": False, "print_csv_filename": False})
                 self._last_model = model
@@ -265,23 +320,26 @@ class LQPExecutor(e.Executor):
             }
             result, final_model = self.compiler.compile_inner(query, options)
             export_info, query_txn = result
+            query_txn = txn_with_configure(query_txn, configure)
             compile_span["compile_type"] = "query"
             compile_span["lqp"] = lqp_print.to_string(query_txn, {"print_names": True, "print_debug": False, "print_csv_filename": False})
 
-        txn = query_txn
-        if model_txn is not None:
-            # Merge the two LQP transactions into one. Long term the query bits should all
-            # go into a WhatIf action. But for now we just use two separate epochs.
-            model_epoch = model_txn.epochs[0]
-            query_epoch = query_txn.epochs[0]
-            txn = lqp_ir.Transaction(
-                epochs=[model_epoch, query_epoch],
-                configure=construct_configure({}, None),
-                meta=None,
-            )
+        # Merge the epochs into a single transactions. Long term the query bits should all
+        # go into a WhatIf action and the intrinsics could be fused with either of them. But
+        # for now we just use separate epochs.
+        epochs = []
 
-            # Revalidate now that we've joined two epochs
-            validate_lqp(txn)
+        epochs.append(self._compile_intrinsics())
+
+        if model_txn is not None:
+            epochs.append(model_txn.epochs[0])
+
+        epochs.append(query_txn.epochs[0])
+
+        txn = lqp_ir.Transaction(epochs=epochs, configure=configure, meta=None)
+
+        # Revalidate now that we've joined all the epochs.
+        validate_lqp(txn)
 
         txn_proto = convert_transaction(txn)
         # TODO (azreika): Should export_info be encoded as part of the txn_proto? [RAI-40312]
@@ -294,6 +352,10 @@ class LQPExecutor(e.Executor):
         df, errs = result_helpers.format_results(raw_results, cols)
         self.report_errors(errs)
 
+        # Rename columns if wide outputs is enabled
+        if self.wide_outputs and len(cols) - len(extra_cols) == len(df.columns):
+            df.columns = cols[: len(df.columns)]
+
         # Process exports
         if export_to and not self.dry_run:
             assert cols, "No columns found in the output"
@@ -310,14 +372,23 @@ class LQPExecutor(e.Executor):
 
         return self._postprocess_df(self.config, df, extra_cols)
 
-    def execute(self, model: ir.Model, task:ir.Task, result_cols:Optional[list[str]]=None, export_to:Optional[str]=None, update:bool=False) -> DataFrame:
+    def execute(self, model: ir.Model, task: ir.Task, format: Literal["pandas", "snowpark"] = "pandas",
+                result_cols: Optional[list[str]] = None, export_to: Optional[str] = None,
+                update: bool = False, meta: dict[str, Any] | None = None) -> DataFrame:
         self.prepare_data()
+        previous_model = self._last_model
 
         final_model, export_info, txn_proto = self.compile_lqp(model, task)
 
         if self.dry_run:
             return DataFrame()
 
+        if format != "pandas":
+            raise ValueError(f"Unsupported format: {format}")
+
+        # Format meta as headers
+        json_meta = prepare_metadata_for_headers(meta)
+        headers = {QUERY_ATTRIBUTES_HEADER: json_meta} if json_meta else {}
         raw_results = self.resources.exec_lqp(
             self.database,
             self.engine,
@@ -327,7 +398,22 @@ class LQPExecutor(e.Executor):
             # transactions are serialized.
             readonly=False,
             nowait_durable=True,
+            headers=headers,
         )
         assert isinstance(raw_results, TransactionAsyncResponse)
 
-        return self._process_results(task, final_model, raw_results, result_cols, export_info, export_to, update)
+        try:
+            return self._process_results(task, final_model, raw_results, result_cols, export_info, export_to, update)
+        except Exception as e:
+            # If processing the results failed, revert to the previous model.
+            self._last_model = previous_model
+            raise e
+
+def txn_with_configure(txn: lqp_ir.Transaction, configure: lqp_ir.Configure) -> lqp_ir.Transaction:
+    """ Return a new transaction with the given configure. If the transaction already has
+    a configure, it is replaced. """
+    return lqp_ir.Transaction(
+        epochs=txn.epochs,
+        configure=configure,
+        meta=txn.meta,
+    )
