@@ -13,7 +13,7 @@ from relationalai.semantics.lqp.utils import TranslationCtx, gen_unique_var
 from relationalai.semantics.lqp.validators import assert_valid_input
 
 from decimal import Decimal as PyDecimal
-from datetime import datetime, date
+from datetime import datetime, date, timezone
 from typing import Tuple, cast, Union, Optional
 from warnings import warn
 import re
@@ -25,10 +25,6 @@ def to_lqp(model: ir.Model, fragment_name: bytes, ctx: TranslationCtx) -> tuple[
     decls: list[lqp.Declaration] = []
     reads: list[lqp.Read] = []
 
-    # Add pyrel error attributes to reads.
-    for err_id in _extract_pyrel_error_ids(ctx, model):
-        ctx.output_ids.append(err_id)
-
     # LQP only accepts logical tasks
     # These are asserted at init time
     root = cast(ir.Logical, model.root)
@@ -37,6 +33,10 @@ def to_lqp(model: ir.Model, fragment_name: bytes, ctx: TranslationCtx) -> tuple[
         new_decls = _translate_to_decls(ctx, subtask)
         decls.extend(new_decls)
 
+    # Add pyrel error attributes to reads.
+    for err_id in _extract_pyrel_error_ids(ctx, model):
+        ctx.output_ids.append(err_id)
+
     reads.extend(_get_output_reads(ctx.output_ids))
 
     export_info = None
@@ -102,6 +102,7 @@ def _get_export_reads(export_ids: list[tuple[lqp.RelationId, int, lqp.Type]]) ->
         data_columns=csv_columns,
         compression="gzip",
         partition_size=200,
+        syntax_escapechar='"',  # To follow Snowflake's expected format
         meta=None,
     )
     reads.append(lqp.Read(read_type=lqp.Export(config=export_csv_config, meta=None), meta=None))
@@ -192,12 +193,21 @@ def _translate_effect(ctx: TranslationCtx, effect: Union[ir.Output, ir.Update],
     elif isinstance(effect, ir.Output):
         ctx.output_ids.append((rel_id, def_name))
 
+    # First we collect annotations on the effect itself, e.g. from something like
+    # `select(...).annotate(...)`.
+    annotations = effect.annotations
+    if isinstance(effect, ir.Update):
+        # Then we translate annotations on the relation itself, e.g.
+        # ```
+        # Bar.foo = model.Relationship(...)
+        # Bar.foo.annotate(...)
+        # ```
+        annotations = annotations | effect.relation.annotations
+
     return lqp.Def(
         name = rel_id,
         body = mk_abstraction(projection, new_body),
-        # TODO this only covers the annotations on the effect itself. Annotations on the
-        # relation are not included yet.
-        attrs = _translate_annotations(effect.annotations),
+        attrs = _translate_annotations(annotations),
         meta = None,
     )
 
@@ -531,6 +541,9 @@ def _translate_to_atom(ctx: TranslationCtx, task: ir.Lookup) -> lqp.Formula:
             terms.append(term)
         return lqp.Cast(input=terms[0], result=terms[1], meta=None)
 
+    if task.relation == builtins.join:
+        return _translate_join(ctx, task)
+
     terms = []
     term_types = []
     for arg in task.args:
@@ -560,16 +573,16 @@ def _translate_to_atom(ctx: TranslationCtx, task: ir.Lookup) -> lqp.Formula:
     return lqp.Atom(name=rid, terms=terms, meta=None)
 
 
-def get_relation_id(ctx: TranslationCtx, relation: ir.Relation, projection: list[Tuple[lqp.Var, lqp.Type]] = [], prefix: Union[str, None] = None) -> lqp.RelationId:
+def get_relation_id(ctx: TranslationCtx, relation: ir.Relation, projection: list[Tuple[lqp.Var, lqp.Type]] = []) -> lqp.RelationId:
     types = "_".join([str(t.type_name) for (_, t) in projection])
     if types:
         types = "_" + types
     if relation.id in ctx.def_names.id_to_name:
         unique_name = ctx.def_names.id_to_name[relation.id]
     else:
-        if prefix is None:
-            prefix = helpers.relation_name_prefix(relation)
-        unique_name = ctx.def_names.get_name_by_id(relation.id, prefix + relation.name)
+        name = helpers.relation_name_prefix(relation) + relation.name
+        name = helpers.sanitize(name)
+        unique_name = ctx.def_names.get_name_by_id(relation.id, name)
 
     return utils.gen_rel_id(ctx, unique_name, types)
 
@@ -625,7 +638,8 @@ def to_lqp_value(value: ir.PyValue, value_type: ir.Type) -> lqp.Value:
     elif typ.type_name == lqp.TypeName.DATE and isinstance(value, date):
         val = lqp.DateValue(value=value, meta=None)
     elif typ.type_name == lqp.TypeName.DATETIME and isinstance(value, datetime):
-        val = lqp.DateTimeValue(value=value, meta=None)
+        utc_value = value.astimezone(timezone.utc) if value.tzinfo is not None else value # Convert to UTC cf. Iceberg
+        val = lqp.DateTimeValue(value=utc_value, meta=None)
     elif typ.type_name == lqp.TypeName.BOOLEAN and isinstance(value, bool):
         val = lqp.BooleanValue(value=value, meta=None)
     else:
@@ -646,9 +660,52 @@ def _extract_pyrel_error_ids(ctx: TranslationCtx, model: ir.Model) -> list[Tuple
     for effect in effects:
         if "pyrel_error_attrs" not in effect.relation.name:
             continue
-        rel_id = get_relation_id(ctx, effect.relation, [], "")
+        projection, _ = _translate_bindings(ctx, _effect_bindings(effect))
+        rel_id = get_relation_id(ctx, effect.relation, projection)
         name = f"{effect.relation.name}_{i}"
         pyrel_error_attrs.append((rel_id, name))
         i += 1
 
     return pyrel_error_attrs
+
+# Hard-coded implementation of Rel's string_join
+def _translate_join(ctx: TranslationCtx, task: ir.Lookup) -> lqp.Formula:
+    assert len(task.args) == 3
+    (strs, separator, target) = task.args
+    assert isinstance(separator, ir.Node)
+    assert isinstance(target, ir.Var)
+    assert isinstance(strs, tuple)
+
+    # Enumerate the strings we're going to be reducing over
+    enumerated_conjunctions = []
+    index_var, index_type = gen_unique_var(ctx, "idx"), meta_type_to_lqp(types.Int64)
+    string_var, string_type = gen_unique_var(ctx, "s"), meta_type_to_lqp(types.String)
+    for i, s in enumerate(strs):
+        index = to_lqp_value(i, types.Int64)
+        string = _translate_term(ctx, s)[0]
+        eq_idx = mk_primitive("rel_primitive_eq", [index_var, index])
+        eq_string = mk_primitive("rel_primitive_eq", [string_var, string])
+        enumerated_conjunctions.append(mk_and([eq_idx, eq_string]))
+    body = mk_abstraction([(index_var, index_type), (string_var, string_type)], mk_or(enumerated_conjunctions))
+
+    # Make the function we're reducing
+    a_var, a_type = gen_unique_var(ctx, "a"), meta_type_to_lqp(types.String)
+    b_var, b_type = gen_unique_var(ctx, "b"), meta_type_to_lqp(types.String)
+    curr_var, curr_type = gen_unique_var(ctx, "curr"), meta_type_to_lqp(types.String)
+    res_var, res_type = gen_unique_var(ctx, "res"), meta_type_to_lqp(types.String)
+
+    sep, _ = _translate_term(ctx, separator)
+
+    op = mk_abstraction(
+        [(a_var, a_type), (b_var, b_type), (curr_var, curr_type)],
+        mk_exists([(res_var, res_type)],
+            mk_and([
+                mk_primitive("rel_primitive_concat", [res_var, b_var, curr_var]),
+                mk_primitive("rel_primitive_concat", [a_var, sep, res_var])
+            ])
+        )
+    )
+
+    output_term = _translate_term(ctx, target)[0]
+
+    return lqp.Reduce(meta=None, op=op, body=body, terms=[output_term])
