@@ -2,13 +2,12 @@ from relationalai.semantics.metamodel.compiler import Pass
 from relationalai.semantics.metamodel import ir, builtins as rel_builtins, factory as f, visitor
 from relationalai.semantics.metamodel.typer import Checker, InferTypes, typer
 from relationalai.semantics.metamodel import helpers, types
-from relationalai.semantics.metamodel.rewrite import (Splinter, ExtractNestedLogicals, ExtractKeys, FDConstraints,
-                                                      DNFUnionSplitter, DischargeConstraints)
 from relationalai.semantics.metamodel.util import FrozenOrderedSet
 
 from relationalai.semantics.metamodel.rewrite import Flatten
-# TODO: Move this into metamodel.rewrite
-from relationalai.semantics.rel.rewrite import QuantifyVars, CDC
+
+from ..metamodel.rewrite import DischargeConstraints, DNFUnionSplitter, ExtractNestedLogicals
+from .rewrite import CDC, ExtractCommon, ExtractKeys, FDConstraints, QuantifyVars, Splinter
 
 from relationalai.semantics.lqp.utils import output_names
 
@@ -25,8 +24,7 @@ def lqp_passes() -> list[Pass]:
         InferTypes(),
         DNFUnionSplitter(),
         ExtractKeys(),
-        # Broken
-        # ExtractCommon(),
+        ExtractCommon(),
         Flatten(),
         Splinter(), # Splits multi-headed rules into multiple rules
         QuantifyVars(), # Adds missing existentials
@@ -548,7 +546,7 @@ class PeriodMath(Pass):
                 return node
 
             if node.relation.name not in {
-                "year", "month", "week", "day", "hour", "minute", "second", "millisecond"
+                "year", "month", "week", "day", "hour", "minute", "second", "millisecond", "microsecond", "nanosecond"
             }:
                 return node
 
