@@ -558,22 +558,23 @@ class Flatten(Pass):
     def rewrite_wide_output(self, output: ir.Output):
         assert(output.keys)
 
-        # only prefix keys that are not already in the output
-        prefix_keys = []
+        # only append keys that are not already in the output
+        suffix_keys = []
         for key in output.keys:
             if all([val is not key for _, val in output.aliases]):
-                prefix_keys.append(key)
+                suffix_keys.append(key)
 
         aliases: OrderedSet[Tuple[str, ir.Value]] = ordered_set()
-        # add the keys to the output
-        for key in prefix_keys:
-            aliases.add((key.name, key))
 
         # add the remaining args, unless it is already a key
         for name, val in output.aliases:
-            if not isinstance(val, ir.Var) or val not in prefix_keys:
+            if not isinstance(val, ir.Var) or val not in suffix_keys:
                 aliases.add((name, val))
 
+        # add the keys to the output
+        for key in suffix_keys:
+            aliases.add((key.name, key))
+
         # TODO - we are assuming that the Rel compiler will translate nullable lookups
         # properly, returning a `Missing` if necessary, like this:
         # (nested_192(_adult, _adult_name) or (not nested_192(_adult, _) and _adult_name = Missing)) and
