@@ -72,6 +72,21 @@ CONVERSION_LATTICE:dict[tuple[ir.Type, ir.Type], int] = {
     (types.Int128, types.UInt128): 1001,
 }
 
+def conversion_lattice_cost(from_type, to_type):
+    """
+    Lookup the conversion cost in the conversion lattice to convert from_type to to_type.
+    Return inf is the conversion is not allowed.
+    """
+     # the conversion lattice only has generic decimals as placeholders
+    if isinstance(from_type, ir.DecimalType):
+        from_type = types.GenericDecimal
+    if isinstance(to_type, ir.DecimalType):
+        to_type = types.GenericDecimal
+
+    if (from_type, to_type) in CONVERSION_LATTICE:
+        return CONVERSION_LATTICE[(from_type, to_type)]
+    return float("inf")
+
 # list of non-parametric types that are primitives. Avoid using this constant, use
 # is_base_primitive instead because it accounts for parametric types like decimals.
 _NON_PARAMETRIC_PRIMITIVES = [
@@ -205,16 +220,8 @@ def conversion_cost(arg: ir.Value, actual:ir.Type, expected:ir.Type, arg_types:s
     if type_matches(base_actual, base_expected):
         return 0
 
-    # the conversion lattice only has generic decimals as placeholders
-    if isinstance(base_actual, ir.DecimalType):
-        base_actual = types.GenericDecimal
-    if isinstance(base_expected, ir.DecimalType):
-        base_expected = types.GenericDecimal
+    return conversion_lattice_cost(base_actual, base_expected)
 
-    if (base_actual, base_expected) in CONVERSION_LATTICE:
-        return CONVERSION_LATTICE[(base_actual, base_expected)]
-
-    return float("inf")
 
 def merge_types(type1: ir.Type, type2: ir.Type, is_literal1:bool, is_literal2:bool) -> ir.Type:
     if type1 == type2:
@@ -402,7 +409,7 @@ class PropagationNetwork():
         # if the node resolves to an overload the uses GenericDecimal, the concrete
         # DecimalType to be used (assumes there's a single overload with decimals for a node)
         self.resolved_overload_decimal:dict[int, ir.DecimalType] = {}
-        self.node_is_literal:dict[Node, bool] = {}
+        self.node_is_literal:dict[int, bool] = {}
 
     #--------------------------------------------------
     # Mismatches
@@ -448,13 +455,13 @@ class PropagationNetwork():
                 # replace it; this is safe because literals are only part of one task
                 self.resolved_types[node.id] = type
             else:
-                self.resolved_types[node.id] = merge_types(self.resolved_types[node.id], type, self.node_is_literal[node], is_literal)
+                self.resolved_types[node.id] = merge_types(self.resolved_types[node.id], type, self.node_is_literal[node.id], is_literal)
         else:
             self.resolved_types[node.id] = type
 
-        if node in self.node_is_literal:
-            is_literal &= self.node_is_literal[node]
-        self.node_is_literal[node] = is_literal
+        if node.id in self.node_is_literal:
+            is_literal &= self.node_is_literal[node.id]
+        self.node_is_literal[node.id] = is_literal
 
     def add_type_requirement(self, source:Node, field:ir.Field):
         self.type_requirements[source].add(field)
@@ -583,7 +590,7 @@ class PropagationNetwork():
             source = work_list.pop(0)
             unhandled.remove(source)
             source_type = self.resolve(source)
-            is_literal = self.node_is_literal.get(source, False)
+            is_literal = self.node_is_literal.get(source.id, False)
             # check to see if the source has ended up with a set of types that
             # aren't valid, e.g. a union of primitives
             if invalid_type(source_type):
@@ -620,7 +627,7 @@ class PropagationNetwork():
                 if not type_matches(node_type, field_type):
                     node_base = to_base_primitive(node_type)
                     field_base = to_base_primitive(field_type)
-                    if (node_base, field_base) not in CONVERSION_LATTICE:
+                    if conversion_lattice_cost(node_base, field_base) == float("inf"):
                         self.mismatch(node, field_type, node_type)
 
         # return the resolved type for fields only
