@@ -2,12 +2,15 @@ from __future__ import annotations
 from typing import Union
 import textwrap
 import uuid
+import time
 
 from relationalai.semantics.snowflake import Table
 from relationalai.semantics import std
 from relationalai.semantics.internal import internal as b # TODO(coey) change b name or remove b.?
 from relationalai.semantics.rel.executor import RelExecutor
 from relationalai.semantics.lqp.executor import LQPExecutor
+from relationalai.tools.constants import DEFAULT_QUERY_TIMEOUT_MINS
+from relationalai.util.timeout import calc_remaining_timeout_minutes
 
 from .common import make_name
 from relationalai.experimental.solvers import Solver
@@ -243,6 +246,17 @@ class SolverModelDev:
         app_name = resources.get_app_name()
         print(app_name)
 
+        # Note: currently the query timeout is not propagated to the steps 'export model
+        # relations', and 'import result relations'. For those steps the default query
+        # timeout value defined in the config will apply.
+        # TODO: propagate the query timeout to those steps as well.
+        query_timeout_mins = kwargs.get("query_timeout_mins", None)
+        config = self._model._config
+        if query_timeout_mins is None and (timeout_value := config.get("query_timeout_mins", DEFAULT_QUERY_TIMEOUT_MINS)) is not None:
+            query_timeout_mins = int(timeout_value)
+        config_file_path = getattr(config, 'file_path', None)
+        start_time = time.monotonic()
+
         # 1. export model relations
         print("export model relations")
         # TODO(coey) perf: only export the relations that are actually used in the model
@@ -266,6 +280,9 @@ class SolverModelDev:
             b.select(*rel._field_refs).where(rel(*rel._field_refs)).into(table)
 
         # 2. execute solver job and wait for completion
+        remaining_timeout_minutes = calc_remaining_timeout_minutes(
+            start_time, query_timeout_mins, config_file_path=config_file_path,
+        )
         print("execute solver job")
         payload = {
             "solver": solver.solver_name.lower(),
@@ -273,7 +290,9 @@ class SolverModelDev:
             "input_id": input_id,
             "data_type": self._data_type
         }
-        job_id = solver._exec_job(payload, log_to_console=log_to_console)
+        job_id = solver._exec_job(
+            payload, log_to_console=log_to_console, query_timeout_mins=remaining_timeout_minutes,
+        )
         print(f"job id: {job_id}") # TODO(coey) maybe job_id is not useful
 
         # 3. import result relations
