@@ -2,12 +2,15 @@ from __future__ import annotations
 from typing import Any, Union
 import textwrap
 import uuid
+import time
 
 from relationalai.semantics.metamodel.util import ordered_set
 from relationalai.semantics.internal import internal as b # TODO(coey) change b name or remove b.?
 from relationalai.semantics.rel.executor import RelExecutor
 from .common import make_name
 from relationalai.experimental.solvers import Solver
+from relationalai.tools.constants import DEFAULT_QUERY_TIMEOUT_MINS
+from relationalai.util.timeout import calc_remaining_timeout_minutes
 
 _Any = Union[b.Producer, str, float, int]
 _Number = Union[b.Producer, float, int]
@@ -222,6 +225,14 @@ class SolverModelPB:
         assert isinstance(executor, RelExecutor)
         prefix_l = f"solvermodel_{self._id}_"
 
+        query_timeout_mins = kwargs.get("query_timeout_mins", None)
+        config = self._model._config
+        if query_timeout_mins is None and (timeout_value := config.get("query_timeout_mins", DEFAULT_QUERY_TIMEOUT_MINS)) is not None:
+            query_timeout_mins = int(timeout_value)
+        config_file_path = getattr(config, 'file_path', None)
+        start_time = time.monotonic()
+        remaining_timeout_minutes = query_timeout_mins
+
         # 1. Materialize the model and store it.
         print("export model")
         b.select(b.count(self.Variable)).to_df() # TODO(coey) weird hack to avoid uninitialized properties error
@@ -244,14 +255,22 @@ class SolverModelPB:
         def config[:envelope, :payload, :data]: model_string
         def config[:envelope, :payload, :path]: "{model_uri}"
         def export {{ config }}
-        """))
+        """), query_timeout_mins=remaining_timeout_minutes)
 
         # 2. Execute job and wait for completion.
         print("execute solver job")
-        job_id = solver._exec_job(payload, log_to_console=log_to_console)
+        remaining_timeout_minutes = calc_remaining_timeout_minutes(
+            start_time, query_timeout_mins, config_file_path=config_file_path,
+        )
+        job_id = solver._exec_job(
+            payload, log_to_console=log_to_console, query_timeout_mins=remaining_timeout_minutes,
+        )
 
         # 3. Extract result.
         print("extract result")
+        remaining_timeout_minutes = calc_remaining_timeout_minutes(
+            start_time, query_timeout_mins, config_file_path=config_file_path,
+        )
         extract_str = textwrap.dedent(f"""
         def raw_result {{
             load_binary["snowflake://APP_STATE.RAI_INTERNAL_STAGE/job-results/{job_id}/result.binpb"]
@@ -289,7 +308,9 @@ class SolverModelPB:
                     ::std::mirror::convert(std::mirror::typeof[Int128], j, i)
                 )
             """)
-        executor.execute_raw(extract_str, readonly=False)
+        executor.execute_raw(
+            extract_str, readonly=False, query_timeout_mins=remaining_timeout_minutes,
+        )
 
         print("finished solve")
         return None
