@@ -2,6 +2,7 @@
 from __future__ import annotations
 from relationalai.semantics.metamodel import types, factory as f
 from relationalai.semantics.metamodel.util import OrderedSet
+from relationalai.semantics.metamodel import builtins
 
 # Rel Annotations as IR Relations (to be used in IR Annotations)
 
@@ -28,7 +29,10 @@ inner_loop_non_stratified_annotation = f.annotation(inner_loop_non_stratified, [
 
 # collect all supported builtin rel annotations
 builtin_annotations = OrderedSet.from_iterable([
-    arrow, no_diagnostics, no_inline, function, inner_loop, inner_loop_non_stratified
+    arrow, no_diagnostics, no_inline, function, inner_loop, inner_loop_non_stratified,
+    # track annotations on relations do not currently propagate into Rel
+    # TODO: from Thiago, ensure annotation goes from the Logical into the proper declaration
+    builtins.track,
 ])
 
 builtin_annotation_names = OrderedSet.from_iterable([a.name for a in builtin_annotations])
