@@ -6,13 +6,13 @@ from decimal import Decimal as PyDecimal
 from relationalai.semantics.metamodel import ir, compiler as c, builtins as bt, types, visitor, helpers, factory as f
 from relationalai.semantics.metamodel.typer import Checker, InferTypes
 from relationalai.semantics.metamodel.typer.typer import to_base_primitive, to_type, _NON_PARAMETRIC_PRIMITIVES
-from relationalai.semantics.metamodel.rewrite import (Flatten, ExtractKeys, FDConstraints, Splinter,
-                                                      ExtractNestedLogicals, DNFUnionSplitter, DischargeConstraints)
 from relationalai.semantics.metamodel.visitor import ReadWriteVisitor
 from relationalai.semantics.metamodel.util import OrderedSet, group_by, NameCache, ordered_set
 
 from relationalai.semantics.rel import rel, rel_utils as u, builtins as rel_bt
-from relationalai.semantics.rel.rewrite import CDC, QuantifyVars
+
+from ..metamodel.rewrite import (Flatten, ExtractNestedLogicals, DNFUnionSplitter, DischargeConstraints)
+from ..lqp.rewrite import CDC, ExtractCommon, ExtractKeys, FDConstraints, QuantifyVars, Splinter
 
 import math
 
@@ -32,7 +32,7 @@ class Compiler(c.Compiler):
             InferTypes(),
             DNFUnionSplitter(),
             ExtractKeys(),
-            # rewrite.ExtractCommon(),
+            ExtractCommon(),
             Flatten(),
             Splinter(),
             QuantifyVars(),
@@ -125,21 +125,6 @@ class ModelToRel:
                 tuple([rel.Annotation("inline", ())]),
             ))
 
-        if "pyrel_num_chars" in reads:
-            defs.append(
-                rel.Def("pyrel_num_chars",
-                    tuple([rel.Var("x"), rel.Var("y")]),
-                    rel.Exists(
-                        tuple([rel.Var("z")]),
-                        rel.And(ordered_set(
-                            rel.atom("::std::common::num_chars", tuple([rel.Var("x"), rel.Var("z")])),
-                            rel.Atom(self._convert_abs(types.Int64, types.Int128), tuple([rel.Var("z"), rel.Var("y")])),
-                        )),
-                    ),
-                    tuple([rel.Annotation("inline", ())]),
-                ),
-            )
-
         if "pyrel_count" in reads:
             defs.append(
                 rel.Def("pyrel_count",
@@ -249,6 +234,9 @@ class ModelToRel:
                     ),
             )
 
+        if "pyrel_regex_search" in reads:
+            raise NotImplementedError("pyrel_regex_search is not implemented")
+
         return defs
 
     @staticmethod
