@@ -150,8 +150,7 @@ class Compute:
             """
             self._graph.model.install_raw(query)
             self._full_reachable_from_installed[key] = True
-        reachable = self._lib.full_reachable_from(self._graph, unwrap(node))
-        return self._lookup(reachable)
+        return self._lib.full_reachable_from(self._graph, node)
 
     def is_reachable(self, node1: dsl.Producer, node2: dsl.Producer) -> dsl.Expression:
         return self._lib.reachable_from(self._graph, unwrap(node1), unwrap(node2))
