@@ -16,8 +16,10 @@ from InquirerPy.base.control import Choice
 from relationalai.clients.util import IdentityParser
 from .cli_controls import divider, Spinner
 from . import cli_controls as controls
-from relationalai.clients import azure
-from typing import Sequence, cast, Any, List
+from typing import Sequence, cast, Any, List, TYPE_CHECKING
+
+if TYPE_CHECKING:
+    from relationalai.clients import azure
 from relationalai.errors import RAIException
 from relationalai.loaders.types import LoadType, UnsupportedTypeError
 from relationalai.loaders.csv import CSVLoader
@@ -717,7 +719,10 @@ def config_check(all_profiles:bool=False):
 @cli.command(help="Print version info")
 def version():
     from .. import __version__
-    from railib import __version__ as railib_version
+    try:
+        from railib import __version__ as railib_version
+    except Exception:
+        railib_version = None
 
     table = Table(show_header=False, border_style="dim", header_style="bold", box=rich_box.SIMPLE)
     def print_version(name, version, latest=None):
@@ -728,7 +733,8 @@ def version():
 
     divider()
     print_version("RelationalAI", __version__, latest_version("relationalai"))
-    print_version("Rai-sdk", railib_version, latest_version("rai-sdk"))
+    if railib_version is not None:
+        print_version("Rai-sdk", railib_version, latest_version("rai-sdk"))
     print_version("Python", sys.version.split()[0])
 
     app_version = None
@@ -1150,9 +1156,14 @@ def import_source_flow(provider: ResourcesBase) -> Sequence[ImportSource]:
 
     if isinstance(provider, snowflake.Resources):
         return snowflake_import_source_flow(provider)
-    elif isinstance(provider, azure.Resources):
-        return azure_import_source_flow(provider)
     else:
+        # Lazy import for azure to avoid optional dependency issues
+        try:
+            from relationalai.clients import azure
+            if isinstance(provider, azure.Resources):
+                return azure_import_source_flow(provider)
+        except ImportError:
+            pass
         raise Exception(f"No import source flow available for {provider_type.__module__}.{provider_type.__name__}")
 
 def snowflake_import_source_flow(provider: snowflake.Resources) -> Sequence[ImportSource]:
